﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MonitoredUndo;

namespace PhotoTagger.Model
{
    public partial class Photo : ISupportsUndo
    {

        #region Properties

        public string FullTitle
        {
            get
            {
                if (IsFavorite)
                    return this.Title + " [Favorite]";
                else
                    return this.Title;
            }
        }

        public bool IsFavorite
        {
            get
            {
                return this.Rating >= 3;
            }
        }

        #endregion
        #region Partials

        /// <summary>
        /// Ensure that the rating is within a valid range.
        /// </summary>
        /// <param name="value">The value of the rating. This value has not yet been applied to the model.</param>
        partial void OnRatingChanging(byte? value)
        {
            if (value < 0 || value > 5)
                throw new ArgumentOutOfRangeException("Rating", "Rating must be between 0 and 5, inclusive.");
        }

        /// <summary>
        /// FullTitle is dependent on the Title property. Raise PropertyChanged("FullTitle") when Title changes.
        /// </summary>
        /// <param name="value">The new Title.</param>
        partial void OnTitleChanged(string value)
        {
            OnPropertyChanged("FullTitle");
        }

        /// <summary>
        /// FullTitle and IsFavorite are dependent on the Rating property. Raise PropertyChanged("FullTitle") when Rating changes.
        /// </summary>
        /// <param name="value">The new Rating.</param>
        partial void OnRatingChanged(byte? value)
        {
            OnPropertyChanged("FullTitle");
            OnPropertyChanged("IsFavorite");
        }

        #endregion
        #region ISupportsUndo Members

        /// <summary>
        /// Return a reference to the object that is the "root" of the object hierarchy.
        /// </summary>
        public object GetUndoRoot()
        {
            return this.Catalog;
        }

        #endregion
    }
}
