﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PhotoTagger.Model;
using System.Data.Objects;
using System.Xml;
using System.Runtime.Serialization;
using System.IO;
using System.Reflection;

namespace PhotoTagger.Repository.XML
{
    public class PhotoRepositoryXML : IPhotoRepository
    {

        #region Members

        private bool _UseEmbeddedResource = false;

        #endregion
        #region Constructors

        public PhotoRepositoryXML()
        {
        }

        public PhotoRepositoryXML(bool useSampleData)
        {
            _UseEmbeddedResource = true;
        }

        #endregion
        #region IPhotoRepository Members

        public Catalog GetCatalog(string catalogName)
        {
            if (_UseEmbeddedResource)
            {
                return Deserialize(Properties.Resources.Main);
            }
            

            var catalogPath = GetCatalogPath(catalogName);
            if (File.Exists(catalogPath))
            {
                return Deserialize(File.ReadAllText(catalogPath));
            }
            else
            {
                return null;
            }
        }


        public void Save(Catalog catalog)
        {
            if (_UseEmbeddedResource)
            {
                throw new InvalidOperationException("Unable to save back to embedded resources.");
            }


            var catalogPath = GetCatalogPath(catalog.Name);

            EnsureDirectoryExists(catalogPath);

            if (File.Exists(catalogPath))
            {
                File.Delete(catalogPath);
            }

            var data = Serialize(catalog);
            File.WriteAllText(catalogPath, data);
        }


        #endregion
        #region Internal

        private static string GetCatalogPath(string catalogName)
        {
            var path = Assembly.GetExecutingAssembly().Location;
            var dir = Path.GetDirectoryName(path);

            var catalogPath = Path.Combine(Path.Combine(dir, "Catalogs"), catalogName + ".xml");
            return catalogPath;
        }

        private void EnsureDirectoryExists(string catalogPath)
        {
            var dir = Path.GetDirectoryName(catalogPath);
            if (!Directory.Exists(dir))
                Directory.CreateDirectory(dir);
        }
        
        private static string Serialize(Catalog instance)
        {
            var sb = new StringBuilder();

            using (var xmlWriter = XmlWriter.Create(sb))
            {
                var ser = new DataContractSerializer(typeof(Catalog));
                ser.WriteObject(xmlWriter, instance);

                xmlWriter.Flush();
                xmlWriter.Close();

                return sb.ToString();
            }
        }

        private static Catalog Deserialize(string instance)
        {
            using (StringReader sr = new StringReader(instance))
            using (XmlReader xr = XmlReader.Create(sr))
            using (XmlDictionaryReader reader = XmlDictionaryReader.CreateDictionaryReader(xr))
            {
                // Deserialize the data and read it from the instance.
                DataContractSerializer ser = new DataContractSerializer(typeof(Catalog));
                var ret = (Catalog)ser.ReadObject(reader, false);

                reader.Close();
                xr.Close();
                sr.Close();

                return ret;
            }
        } 

        #endregion
    }
}
