﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;

namespace PhotoTagger.ViewModels
{
    sealed public class GenericCommand : ICommand
    {
        private readonly Func<object, bool> _canExecuteDelegate;
        private readonly Action<object> _executeDelegate;

        public GenericCommand(Action<object> executeMethod)
            : this(executeMethod, arg => true)
        {

        }

        public GenericCommand(Action<object> executeMethod, Func<object, bool> canExecuteMethod)
        {
            _executeDelegate = executeMethod;
            _canExecuteDelegate = canExecuteMethod;
        }

        void ICommand.Execute(object parameter)
        {
            _executeDelegate(parameter);
        }

        bool ICommand.CanExecute(object parameter)
        {
            return _canExecuteDelegate(parameter);
        }

        event EventHandler ICommand.CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }
    }
}
