﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity;
using System.Windows;
using System.Windows.Input;
using System.Windows.Data;
using System.Windows.Controls;
using System.Collections.ObjectModel;

namespace PhotoTagger.Behaviors
{
    /// <summary>
    /// This is a "Blend style" behavior that will force the currently focussed element to update it's binding.
    /// </summary>
    /// <remarks>
    /// Use this as a triggered action behind a toolbar button click. 
    /// This is helpful / necessary because the toolbar buttons don't transfer focus away from other controls. 
    /// The consequence is that the bindings with a "LostFocus" UpdateSourceTrigger will not save their value to the source.
    /// </remarks>
    public class CommitBindingsBehavior : TargetedTriggerAction<FrameworkElement>
    {

        protected override void Invoke(object parameter)
        {
            if (null == this.Target)
                return;

            CommitBindings(this.Target);
        }

        public static void CommitBindings(FrameworkElement target)
        {
            // Find the element in this form that has focus (if any).
            var focusedElement = VisualTreeSearch.Find(target, t =>
            {
                var elem = t as UIElement;
                return (null != elem && elem.IsKeyboardFocused);
            }
            ) as FrameworkElement;

            if (null == focusedElement)
                return;

            //Data Grid requires special attention
            var par = VisualTreeSearch.FindByParentType(focusedElement, typeof(DataGrid)) as DataGrid;
            if (par != null)
            {
                par.CommitEdit();
                par.CommitEdit(); // Not sure why, but call it twice.
            }

            foreach (var be in DependencyPropertiesHelper.EnumerateAllBindingExpressions(focusedElement))
            {
                if (null != be)
                    be.Item2.UpdateSource();
            }
        }
    }
}
