﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using PhotoTagger.ViewModels;
using MonitoredUndo;
using PhotoTagger.Behaviors;

namespace PhotoTagger
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        #region View Model Accessor

        private MainViewModel VM
        {
            get
            {
                return this.Resources["MainViewModel"] as MainViewModel;
            }
        }

        #endregion
        #region Commands

        private void Undo_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            var catalog = VM.Catalog;

            if (null != catalog)
                UndoService.Current[catalog].Undo();
        }

        private void Redo_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            var catalog = VM.Catalog;

            if (null != catalog)
                UndoService.Current[catalog].Redo();
        }

        private void Open_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            CommitBindings();
            VM.PickFolderCommand.Execute(e.Parameter);
        }

        private void Save_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            CommitBindings();
            VM.SaveCommand.Execute(e.Parameter);
        }

        private void Delete_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            CommitBindings();
            VM.ExcludePhotoCommand.Execute(e.Parameter);
        }

        private void RatingSlider_GotMouseCapture(object sender, MouseEventArgs e)
        {
            if (null != VM.Catalog)
                UndoService.Current[VM.Catalog].BeginChangeSetBatch("Rating Change", true);
        }

        private void RatingSlider_LostMouseCapture(object sender, MouseEventArgs e)
        {
            if (null != VM.Catalog)
                UndoService.Current[VM.Catalog].EndChangeSetBatch();
        }

        #endregion
        #region Internal

        private void CommitBindings()
        {
            CommitBindingsBehavior.CommitBindings(LayoutRoot);
        }

        #endregion
    }
}
