﻿using System;
using System.Windows.Data;
using ViewModelLocatorsWithConverters.Model;
using ViewModelLocatorsWithConverters.ViewModels;

namespace ViewModelLocatorsWithConverters.Locators
{

    /// <summary>
    /// Provides a method and a XAML IValueConverter to get the viewmodel that is associated with the specified model or view model.
    /// </summary>
    public class FamilyViewModelLocator : IValueConverter
    {

        /// <summary>
        /// For the specified parameter, find or create an appropriate viewmodel.
        /// </summary>
        /// <param name="value">An instance of a "parent" view model, or an instance of the domain model class.</param>
        /// <returns>A new, or previously used view model instance.</returns>
        public static FamilyViewModel GetFamilyViewModel(object value)
        {
            if (null == value)
                return null;

            // Check if we already have an instance of the viewmodel.
            // If so, we're good to go.
            var vm = value as FamilyViewModel;

            if (null == vm)
            {
                // The instance of the model that the ViewModel will handle.
                Family model = null;

                // See if the specified parameter is a known type.
                var family = value as Family;

                // Pull the domain model class from which ever instance we were passed.
                if (ViewModelLocator.IsInDesignMode)
                    model = ViewModelLocator.InitDesignTimeInstance();
                else if (null != family)
                    model = family;
                else
                    throw new ArgumentException("Value parameter must be an instance of a Family or FamilyViewModel. Specified value is of type " + (null == value ? "<null>" : value.GetType().AssemblyQualifiedName), "value");

                // Get a cached ViewModel or create a new instance.
                vm = ViewModelLocator.GetViewModel<FamilyViewModel>(model);
                if (null == vm)
                {
                    vm = new FamilyViewModel(model);

                    // Save the VM in the cache. It will be used again if the same Family is shown again.
                    ViewModelLocator.CacheViewModel(model, vm);
                }
            }

            return vm;
        }


        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return GetFamilyViewModel(value);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }


}
