﻿using System;
using System.Windows.Data;
using ViewModelLocatorsWithConverters.ViewModels;

namespace ViewModelLocatorsWithConverters.Locators
{
    /// <summary>
    /// Provides a method and a XAML IValueConverter to get the viewmodel that is associated with the specified model or view model.
    /// </summary>
    public class MainViewModelLocator : IValueConverter
    {

        /// <summary>
        /// For the specified parameter, find or create an appropriate viewmodel.
        /// </summary>
        /// <param name="value">An instance of a "parent" view model, or an instance of the domain model class.</param>
        /// <returns>A new, or previously used view model instance.</returns>
        public static MainViewModel GetMainViewModel(object value)
        {
            // Check if we already have an instance of the viewmodel
            var vm = value as MainViewModel;

            if (null == vm)
            {
                // Get or create the ViewModel
                vm = ViewModelLocator.GetViewModel<MainViewModel>(null);
                if (null == vm)
                {
                    vm = new MainViewModel();

                    // Design time?
                    if (ViewModelLocator.IsInDesignMode)
                    {
                        // Setup the ViewModel for design time.
                        var family = ViewModelLocator.InitDesignTimeInstance();
                        vm.Families.Add(family);
                        vm.CurrentFamily = family;
                    }

                    // Save the VM in the cache. It will be used again if the same Main is shown again.
                    ViewModelLocator.CacheViewModel(null, vm);
                }
            }

            return vm;

        }


        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return GetMainViewModel(value);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
