﻿using System;
using System.Windows.Data;
using ViewModelLocatorsWithConverters.Model;
using ViewModelLocatorsWithConverters.ViewModels;
using System.Globalization;

namespace ViewModelLocatorsWithConverters.Locators
{
    /// <summary>
    /// Provides a method and a XAML IValueConverter to get the viewmodel 
    /// that is associated with the specified model or view model.
    /// </summary>
    public class PersonViewModelLocator : IValueConverter
    {

        /// <summary>
        /// For the specified parameter, find or create an appropriate viewmodel.
        /// </summary>
        /// <param name="value">An instance of a "parent" view model, 
        /// or an instance of the domain model class.</param>
        /// <returns>A new, or previously used view model instance.</returns>
        public static PersonViewModel GetPersonViewModel(object value)
        {
            // Check if we already have an instance of the viewmodel
            var vm = value as PersonViewModel;

            if (null == vm)
            {
                // The instance of the model that the ViewModel will handle.
                Person model = null;

                // See if the specified parameter is a known type.
                var Person = value as Person;

                // Pull the desired domain model class from the parameter.
                if (ViewModelLocator.IsInDesignMode)
                    model = ViewModelLocator.InitDesignTimeInstance().Members[0];
                else if (null != Person)
                    model = Person;
                else
                    throw new ArgumentException("Value parameter must be an instance of a " + 
                        "Person or PersonViewModel. Specified value is of type " + 
                        (null == value ? "<null>" : value.GetType().AssemblyQualifiedName), 
                        "value");

                // Get or create the ViewModel
                vm = ViewModelLocator.GetViewModel<PersonViewModel>(model);
                if (null == vm)
                {
                    vm = new PersonViewModel(model);

                    // Save the VM in the cache. It will be used again if the same Person is shown again.
                    ViewModelLocator.CacheViewModel(model, vm);
                }
            }

            return vm;
        }


        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return GetPersonViewModel(value);
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
