﻿using System;
using System.Linq;
using System.Windows.Data;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;
using ViewModelLocatorsWithConverters.Model;

namespace ViewModelLocatorsWithConverters.ViewModels
{
    public class FamilyViewModel : ViewModelBase
    {

        #region Constructors
        
        public FamilyViewModel(Family model)
        {
            Family = model;
        }

        #endregion

        #region Commands

        public ICommand TogglePetsCommand
        {
            get
            {
                return new RelayCommand(
                        () => this.ShowPets = !this.ShowPets,
                        () => HasPets
                    );
            }
        }

        #endregion

        #region Properties

        private Family _Family;
        public Family Family
        {
            get { return _Family; }
            set
            {
                if (value == _Family)
                    return;

                _Family = value;
                OnPropertyChanged("Family");
            }
        }

        public bool HasPets
        {
            get
            {
                if (null == Family)
                    return false;

                return Family.Members.Any(m => m.Role == "Cat" || m.Role == "Dog");
            }
        }


        private Person _SelectedPerson;
        public Person SelectedPerson
        {
            get { return _SelectedPerson; }
            set
            {
                if (value == _SelectedPerson)
                    return;

                _SelectedPerson = value;
                OnPropertyChanged("SelectedPerson");
            }
        }


               

        private bool _ShowPets = true;
        public bool ShowPets
        {
            get { return _ShowPets; }
            set
            {
                if (value == _ShowPets)
                    return;

                _ShowPets = value;

                var cvs = CollectionViewSource.GetDefaultView(Family.Members);
                if (!_ShowPets)
                    cvs.Filter = new Predicate<object>(o => ((Person)o).Role != "Dog" && ((Person)o).Role != "Cat");
                else
                    cvs.Filter = null;

                OnPropertyChanged("ShowPets");
                OnPropertyChanged("ShowPetsButtonText");
            }
        }


        public string ShowPetsButtonText
        {
            get
            {
                if (!HasPets)
                    return "No Pets";
                else
                    return ShowPets ? "Hide Pets" : "Show Pets";

            }
        }


        #endregion

    }
}
