﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;
using ViewModelLocatorsWithConverters.Model;

namespace ViewModelLocatorsWithConverters.ViewModels
{
    public class MainViewModel : ViewModelBase
    {

        #region Constructors

        public MainViewModel()
        {
            _Families = new ObservableCollection<Family>();
        }

        #endregion

        #region Commands

        public ICommand LoadFamiliesCommand
        {
            get
            {
                return new RelayCommand(
                        () => LoadFamilies(),
                        () => _Families.Count == 0
                    );
            }
        }

        #endregion
        
        #region Properties

        private IList<Family> _Families;
        public IList<Family> Families
        {
            get { return _Families; }
        }


        private Family _CurrentFamily;
        public Family CurrentFamily
        {
            get { return _CurrentFamily; }
            set
            {
                if (value == _CurrentFamily)
                    return;

                _CurrentFamily = value;
                OnPropertyChanged("CurrentFamily");
            }
        }


        #endregion
        
        #region Internal

        private void LoadFamilies()
        {
            Family family = new Family()
            {
                FamilyName = "Doe Family"
            };

            family.Members.Add(new Person()
            {
                Name = "John Doe",
                Role = "Father"
            });

            family.Members.Add(new Person()
            {
                Name = "Jane Doe",
                Role = "Mother"
            });

            family.Members.Add(new Person()
            {
                Name = "Jimmy Doe",
                Role = "Child"
            });

            family.Members.Add(new Person()
            {
                Name = "Snuggles",
                Role = "Cat"
            });

            Families.Add(family);



            family = new Family()
            {
                FamilyName = "Smith Family"
            };

            family.Members.Add(new Person()
            {
                Name = "John Smith",
                Role = "Father"
            });

            family.Members.Add(new Person()
            {
                Name = "Jane Smith",
                Role = "Mother"
            });

            family.Members.Add(new Person()
            {
                Name = "Jimmy Smith",
                Role = "Child"
            });

            family.Members.Add(new Person()
            {
                Name = "Jill Smith",
                Role = "Child"
            });

            family.Members.Add(new Person()
            {
                Name = "Spot",
                Role = "Dog"
            });

            Families.Add(family);



            family = new Family()
            {
                FamilyName = "Bachelor Family"
            };

            family.Members.Add(new Person()
            {
                Name = "Just Me",
                Role = "Bachelor"
            });


            Families.Add(family);



            family = new Family()
            {
                FamilyName = "Yuppie Family"
            };

            family.Members.Add(new Person()
            {
                Name = "Jim Jones",
                Role = "The Man"
            });

            family.Members.Add(new Person()
            {
                Name = "Jill Jones",
                Role = "Diva"
            });

            Families.Add(family);


            CurrentFamily = Families[0];
        }


        #endregion
        
    }
}
